searchNodes=[{"doc":"","ref":"UseCase.html","title":"UseCase","type":"module"},{"doc":"","ref":"UseCase.html#call/1","title":"UseCase.call/1","type":"function"},{"doc":"","ref":"UseCase.html#call/2","title":"UseCase.call/2","type":"function"},{"doc":"","ref":"UseCase.html#call/3","title":"UseCase.call/3","type":"function"},{"doc":"","ref":"UseCase.html#call!/1","title":"UseCase.call!/1","type":"function"},{"doc":"","ref":"UseCase.html#call!/2","title":"UseCase.call!/2","type":"function"},{"doc":"","ref":"UseCase.html#call!/3","title":"UseCase.call!/3","type":"function"},{"doc":"","ref":"UseCase.html#pipe/1","title":"UseCase.pipe/1","type":"function"},{"doc":"","ref":"UseCase.html#pipe/2","title":"UseCase.pipe/2","type":"function"},{"doc":"","ref":"UseCase.html#pipe/3","title":"UseCase.pipe/3","type":"function"},{"doc":"","ref":"UseCase.html#pipe!/1","title":"UseCase.pipe!/1","type":"function"},{"doc":"","ref":"UseCase.html#pipe!/2","title":"UseCase.pipe!/2","type":"function"},{"doc":"","ref":"UseCase.html#pipe!/3","title":"UseCase.pipe!/3","type":"function"},{"doc":"Adds ok/0,ok/1, error/0, error/1 and error/2 functions to a module that uses it","ref":"UseCase.Interactor.html","title":"UseCase.Interactor","type":"module"},{"doc":"UseCase A way to increase Elixir projects readability and maintenance based on Use Cases and Interactors, the main goals are: Single Responsability Principle Composability Readability Screaming architecture Enforce inputs and outputs of our project use cases Better errors (Know exactly where code fails) Standardization","ref":"readme.html","title":"UseCase","type":"extras"},{"doc":"Installation Interactors Defining inputs Defining errors Default fields Composing with pipes Sending Options Contribute","ref":"readme.html#table-of-contents","title":"UseCase - Table of contents","type":"extras"},{"doc":"The package can be installed by adding use_case to your list of dependencies in mix.exs: def deps do [ {:use_case, &quot;~&gt; 0.1.3&quot;} ] end","ref":"readme.html#installation","title":"UseCase - Installation","type":"extras"},{"doc":"The most basic interactor can be created using the UseCase.Interactor module, defining an output for it and creating a call/2 function: defmodule SayHello do use UseCase.Interactor, output: [:message] def call(%{name: name}, _opts), do: ok(message: &quot;Hello \#{name}!&quot;) def call(%{name: nil}, _opts), do: error(&quot;name is obrigatory&quot;) end Now our SayHello module has the ok and error macros and a struct for Output like %SayHello.Output{message: &quot;something&quot;}. The ok and error macro can be used to define when our interactor success or fail. After define, we can call it in many ways: iex&gt; UseCase.call(SayHello, %{name: &quot;Henrique&quot;}) iex&gt; {:ok, SayHello.Output{message: &quot;Hello Henrique!&quot;, _state: nil}} iex&gt; SayHello.call(%{name: &quot;Henrique&quot;}) iex&gt; {:ok, SayHello.Output{message: &quot;Hello Henrique!&quot;, _state: nil}} iex&gt; UseCase.call(SayHello, %{name: nil}) iex&gt; {:error, SayHello.Error{message: &quot;name is obrigatory!&quot;}} iex&gt; UseCase.call!(SayHello, %{name: &quot;Henrique&quot;}) iex&gt; SayHello.Output{message: &quot;Hello Henrique!&quot;, _state: nil} iex&gt; UseCase.call!(SayHello, %{name: nil}) iex&gt; **** SayHello.Error name is obrigatory! Defining inputs Sometimes we want to guarantee the inputs our interactors will receive, we can do it defining this way: defmodule SayHello do use UseCase.Interactor, output: [:message], input: [:name] # Add this def call(%SayHello{name: name}, _opts), do: ok(message: &quot;Hello \#{name}!&quot;) def call(%SayHello{name: nil}, _opts), do: error(&quot;name is obrigatory&quot;) end Now, with UseCase module we can call it using the input directly: iex&gt; UseCase.call %SayHello{name: &quot;Henrique&quot;} iex&gt; {:ok, SayHello.Output{message: &quot;Hello Henrique!&quot;, _state: nil}} iex&gt; UseCase.call! %SayHello{name: &quot;Henrique&quot;} iex&gt; SayHello.Output{message: &quot;Hello Henrique!&quot;, _state: nil} Defining errors If we want to send extra informations in errors, we can do it as input and output. defmodule SayHello do use UseCase.Interactor, output: [:message], input: [:name], error: [:code] # Add this def call(%SayHello{name: name}, _opts), do: ok(message: &quot;Hello \#{name}!&quot;) def call(%SayHello{name: nil}, _opts), do: error(&quot;name is obrigatory&quot;, code: 500) # And use it end iex&gt; UseCase.call(SayHello, %{name: nil}) iex&gt; {:error, SayHello.Error{message: &quot;name is obrigatory!&quot;, code: 500}} Default fields When not defined, input, output and error defaults to: input: [:_state], output: [], error: [:message] Fields :_state in input and :message in error are always appended. The :_state field is very useful for pipe operations. Composing with pipes Lets define an LogOperation interactor: defmodule LogOperation do use UseCase.Interactor def call(%{message: message}, _opts) do # .. log message ok() end end We can compose with our SayHello simple as that: iex&gt; UseCase.pipe [%SayHello{name: &quot;Henrique&quot;}, LogOperation] iex&gt; {:ok, LogOperation.Output{_state: nil}} iex&gt; UseCase.pipe [%SayHello{name: nil}, LogOperation] iex&gt; {:error, SayHello.Error{message: &quot;name is obrigatory!&quot;, code: 500}} iex&gt; UseCase.pipe! [%SayHello{name: &quot;Henrique&quot;}, LogOperation] iex&gt; LogOperation.Output{_state: nil} iex&gt; UseCase.pipe! [%SayHello{name: nil}, LogOperation] iex&gt; **** SayHello.Error name is obrigatory! All we need is match outputs and inputs and use one of pipe UseCase functions. Sending options All UseCase functions last argument is the options keyword list that is sent to interactors: import UseCase call(%SayHello{name: &quot;henrique&quot;}, my_option: true) pipe([%SayHello{name: &quot;Henrique&quot;}, LogOperation], my_option: true)","ref":"readme.html#interactors","title":"UseCase - Interactors","type":"extras"},{"doc":"UseCase is not only for me, but for the Elixir community. I&#39;m totally open to new ideas. Fork, open issues and feel free to contribute with no bureaucracy. We only need to keep some patterns to maintain an organization: branchs your_branch_name commits [your_branch_name] Your commit","ref":"readme.html#contribute","title":"UseCase - Contribute","type":"extras"}]